/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.guard.mail.autocrypt;

import javax.mail.Address;
import javax.mail.MessagingException;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;
import org.json.JSONException;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.openexchange.capabilities.CapabilityService;
import com.openexchange.capabilities.CapabilitySet;
import com.openexchange.exception.OXException;
import com.openexchange.guard.api.GuardApi;
import com.openexchange.guard.api.GuardApis;
import com.openexchange.session.Session;

/**
 * {@link BackendAutoCryptServiceImpl}
 *
 * @author <a href="mailto:greg.hill@open-xchange.com">Greg Hill</a>
 * @since v2.10.2
 */
public class BackendAutoCryptServiceImpl implements BackendAutoCryptService {

    private static final Logger LOG = LoggerFactory.getLogger(BackendAutoCryptServiceImpl.class);

    private final CapabilityService capabilityService;
    private final String GUARD_MAIL_CAPABILITY = "guard-mail";
    private final GuardApi guardApi;

    public BackendAutoCryptServiceImpl (CapabilityService capabilityService, GuardApi guardApi) {
        this.capabilityService = capabilityService;
        this.guardApi = guardApi;
    }

    /**
     * Verify the user has guard-mail capability
     * @param session
     * @return true if capability exists
     * @throws OXException
     */
    private boolean userHasCapability (Session session) throws OXException {
        if (session == null) {
            return false;
        }
        final CapabilitySet capabilities = this.capabilityService.getCapabilities(session);
        return capabilities.contains(GUARD_MAIL_CAPABILITY);
    }

    /* (non-Javadoc)
     * @see com.openexchange.guard.mail.autocrypt.AutoCryptService#updateOutgoing(javax.mail.internet.MimeMessage, com.openexchange.session.Session)
     */
    @Override
    public MimeMessage updateOutgoing(MimeMessage message, Session session) throws OXException {
        if (userHasCapability(session)) {
            try {
                final Address[] fromArray = message.getFrom();
                if (fromArray != null && fromArray.length > 0) {  // requires from address
                    final InternetAddress addr = new InternetAddress(fromArray[0].toString());
                    final String fromAddress = addr.getAddress();
                    final JSONObject response = guardApi.doCallGet(
                        GuardApis.mapFor("action", "getautocrypt",
                        "id", Integer.toString(session.getUserId()),
                        "cid", Integer.toString(session.getContextId()),
                        "email", fromAddress
                        ), JSONObject.class, session);

                    if (response.has("data")) {
                        try {
                            final String header = response.getString("data");
                            if (header != null && !header.isEmpty()) {
                                message.addHeader("Autocrypt", header);
                                message.saveChanges();
                            }
                        } catch (JSONException | MessagingException ex) {
                            LOG.error("Error processing autocrypt key", ex);
                        }
                    }
                }
            // Not critical error if unable to add autocrypt header, but log
            } catch (final Exception ex) {
                LOG.error("Error adding autocrypt header ", ex);
            }

        }
        return message;

    }

}
