/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.guard.mail.service.dataObjects;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import static com.openexchange.java.Autoboxing.l;
import com.openexchange.java.Strings;
import com.openexchange.mail.dataobjects.MailMessage;
import com.openexchange.mail.dataobjects.SecurityResult;
import com.openexchange.mail.dataobjects.SignatureResult;
import com.openexchange.tools.encoding.Base64;

/**
 * {@link CryptoSecurityResult}
 * Contains results of Crypto action
 * Signature data, if decoded
 *
 * @author <a href="mailto:greg.hill@open-xchange.com">Greg Hill</a>
 * @since v2.8.0
 */
public class CryptoSecurityResult extends SecurityResult{

    private static final String VERIFIED_PROPERTY = "verified";
    private static final String DATE_PROPERTY = "signaturecreatedon";
    private static final String MISSING_PROPERTY = "missing";
    private static final String ERROR_PROPERTY = "error";
    private static final String ISSUER_USERIDS_PROPERTY = "issueruserids";
    private static final String ISSUER_KEY_FINGERPRINT_PROPERTY = "issuerkeyfingerprint";
    private static final String ISSUER_KEY_ID_PROPERTY = "issuerkeyid";



    public CryptoSecurityResult(MailMessage msg, EncryptionType type) {
        super();
        this.signatureResults = null;
        addHeaderSignatureResults(msg.getHeader("X-Guard-Signature-Result"));
        msg.removeHeader("X-Guard-Signature-Result");
        if (msg.hasHeaders("X-PGPFormat")) {
            this.pgpInline = msg.getHeader("X-PGPFormat", null).trim().equals("inline");
        }
        this.type = type;
        this.decryptSuccess = false;
        this.error = null;
    }

    public CryptoSecurityResult(EncryptionType type) {
        super();
        this.type = type;
        this.decryptSuccess = false;
        this.signatureResults = null;
        this.error = null;
        this.pgpInline = false;
    }

    /**
     * Internal method parse the user IDs from the given header value
     *
     * @param headerValue The user IDs
     * @return A list containing the parsed user IDs
     */
    private List<String> parseUserIds(String headerValue){
        if(Strings.isNotEmpty(headerValue)) {
            String[] userIds = headerValue.split(",");
            return Arrays.asList(userIds);
        }
        return Collections.emptyList();
    }

    /**
     * Set that this message was decoded
     */
    public void setDecoded() {
        this.decryptSuccess = true;
    }

    /**
     * Failed message
     * @param error
     */
    public void setFail(String error) {
        this.decryptSuccess = false;
        this.error = error;
    }

    /**
     * Create from string of headers
     * @param headers
     */
    public void addHeaderSignatureResults(String [] headers) {
        if (headers == null) {
            return;
        }
        for (String header: headers) {
            addHeaderSignatureResult (header);
        }
    }

    public void addHeaderSignatureResult (String base64header) {
        String header = new String(Base64.decode(base64header));
        String[] lines = header.split(";");
        SignatureResult sig = new SignatureResult();
        for (String line : lines) {
            String [] result = line.split("=");
            if(result.length > 1) {
                switch (result[0].toLowerCase().trim()) {
                    case VERIFIED_PROPERTY:
                       sig.setVerified(result[1].toLowerCase().trim().equals("true"));
                       break;
                    case MISSING_PROPERTY:
                        sig.setMissing(result[1].toLowerCase().trim().equals("true"));
                        break;
                    case DATE_PROPERTY:
                        sig.setDate(Long.parseLong(result[1].trim()));
                        break;
                    case ERROR_PROPERTY:
                        sig.setError(result[1]);
                        break;
                    case ISSUER_USERIDS_PROPERTY:
                        sig.setIssuerUserIds(parseUserIds(result[1]));
                        break;
                    case ISSUER_KEY_FINGERPRINT_PROPERTY:
                        sig.setIssuerKeyFingerprint(result[1]);
                        break;
                    case ISSUER_KEY_ID_PROPERTY:
                        sig.setIssuerKeyId(l(Long.valueOf(result[1])));
                        break;
                }
            }
        }
        if (signatureResults == null) {
            signatureResults = new ArrayList<SignatureResult> ();
        }
        signatureResults.add(sig);
    }


}
