/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.guard.sharing;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import com.openexchange.exception.OXException;
import com.openexchange.file.storage.File;
import com.openexchange.file.storage.File.Field;
import com.openexchange.file.storage.FileStorageFileAccess;
import com.openexchange.file.storage.FileStorageIgnorableVersionFileAccess;
import com.openexchange.file.storage.FileStorageObjectPermission;
import com.openexchange.file.storage.composition.crypto.CryptoAwareSharingService;
import com.openexchange.groupware.ComparedPermissions;
import com.openexchange.guard.exceptions.GuardExceptionCodes;
import com.openexchange.guard.internal.AbstractGuardAccess;
import com.openexchange.guard.internal.GuardApiImpl;
import com.openexchange.guard.internal.authentication.AuthenticationTokenHandler;
import com.openexchange.guard.internal.authentication.GuardAuthenticationToken;
import com.openexchange.session.Session;


/**
 * {@link CryptoAwareSharingServiceImpl}
 *
 * @author <a href="mailto:greg.hill@open-xchange.com">Greg Hill</a>
 * @since v2.8.0
 */
public class CryptoAwareSharingServiceImpl extends AbstractGuardAccess implements CryptoAwareSharingService {

    /**
     * Check if file is encrypted
     * @param file
     * @return
     */
    @Override
    public boolean isEncrypted (File file) {
        return file.getMeta().containsKey("Encrypted") && file.getMeta().get("Encrypted").equals(true);
    }


    /* (non-Javadoc)
     * @see com.openexchange.file.storage.composition.CryptoAwareSharingService#updateSharing(com.openexchange.session.Session, com.openexchange.file.storage.File, com.openexchange.groupware.ComparedPermissions)
     */
    @Override
    public void updateSharing(Session session, File file, FileStorageFileAccess fileAccess, ComparedPermissions permissions, List<FileStorageObjectPermission> updatedPermissions) throws OXException {
        // If no changes in users, then return
        if (!permissions.hasAddedGuests() && !permissions.hasAddedUsers() && !permissions.hasNewGuests() && !permissions.hasRemovedGuests() && !permissions.hasRemovedUsers()) {
            return;
        }
        if (permissions.hasAddedGroups()) {
            throw GuardExceptionCodes.ACTION_NOT_ALLOWED.create("Group sharing");
        }

        AuthenticationTokenHandler authHandler = new AuthenticationTokenHandler();
        GuardAuthenticationToken authToken = authHandler.requireForSession(session);
        InputStream fileData = fileAccess.getDocument(file.getFolderId(), file.getId(), FileStorageFileAccess.CURRENT_VERSION);
        GuardApiImpl guardApi = requireGuardApi(PGPCORE_ENDPOINT);
        GuardSharingModificationStream modifiedStream =
            (updatedPermissions != null) ?
                new GuardSharingModificationStream(guardApi, fileData, updatedPermissions, authToken, session) :
                new GuardSharingModificationStream(guardApi, fileData, permissions, authToken, session);
        try (InputStream input = modifiedStream.getGuardStream()) {
            ((FileStorageIgnorableVersionFileAccess) fileAccess).saveDocument(file,
                input,
                file.getSequenceNumber(),
                new ArrayList<Field>(),
                true);
        } catch (IOException e) {
            throw GuardExceptionCodes.UNKNOWN_ERROR.create("Problem saving file with new share recipients");
        }


    }

}
